
;==========================================================
; objectsDrawAll()
;==========================================================
;draws all objects
objectsDrawAll:
	ld b,18
	ld de,16
	ld ix,dataObjects+32
objectsDrawAll_loop:
	push bc
	push de
	push ix
	ld a,(ix)
	or a \ call nz, objectDraw
	pop ix
	pop de
	add ix,de
	pop bc
	djnz objectsDrawAll_loop
	ret


;==========================================================
; objectDraw(ix=adress)
;==========================================================
;draws object
objectDraw:
	call objectGetScreenPos
	push de

	;==Check for out-of-screen-ness
	ld a,d
	add a,64
	cp 224
	call nc, objectVanish

	ld a,(ix+sort)
	cp GOOMBA \ jr z,drawGoomba
	cp GOOMBA_CORPSE \ jr z,drawGoombaCorps
	cp KOOPA_TROOPA \ jr z,drawKoopaTroopa
	cp KOOPA_SHELL	\ jr z,drawKoopaShell
	cp JUMPY_TILE \ jp z,drawJumpyTile
	cp JUMPY_COIN \ jp z,drawJumpyCoin
	cp MAGIC_MUSHROOM \ jp z,drawMagicMushroom

	pop hl
	ret

drawGoomba:
	ld a,(frameCounter)
	ld hl,Goomba_2
	and %00000100 \ jr nz,drawGoomba_2
	ld hl,Goomba_1
drawGoomba_2:
	pop de
	call ClipSprite_MASK
	ret

drawGoombaCorps:
	ld hl,GoombaCorps
	pop de
	call ClipSprite_MASK
	ret

drawKoopaTroopa:
	ld hl,KoopaTroopa_right1
	ld a,(ix+xspeed)
	or a \ jp p, drawKoopaTroopa_1
	ld hl,KoopaTroopa_left1
drawKoopaTroopa_1:
	ld a,(frameCounter)
	and %00001000 \ jr nz, drawKoopaTroopa_2
	ld de,54
	add hl,de
drawKoopaTroopa_2:
	pop de
	call ClipSprite_MASK
	ret


	.db "dirkdirk"

drawKoopaShell:
	ld hl,KoopaShell_1
	ld a,(ix+xspeed)
	or a \ jr z,dsp_draw
	ld a,(frameCounter)
	and %00001100
	ld b,a
	rr b
	rr b
	add a,a
	add a,a
	add a,b
	ld d,0
	ld e,a
	add hl,de
dsp_draw:
	pop de
	call ClipSprite_MASK
	ret

drawJumpyTile:
	pop de
	inc d
	push de
	ld hl,ErasingSprite
	push ix
	call ClipSprite_MASK
	pop ix
	pop de

	dec e
	dec e
	dec e
	ld a,(ix+state)
	cp 2 \ jr nz,drawj
	dec e
	dec e
drawj:	
	ld a,(ix+origtile)
	ld hl,JumpyQBox
	or a \ jr z,drawj_1
	ld hl,JumpyBricks
drawj_1:
	call ClipSprite_MASK
	ret

drawJumpyCoin:
	ld hl,JumpyCoin1
	ld a,(frameCounter)
	and 1
	jr z,drawJumpyCoin_1
	ld hl,JumpyCoin2
drawJumpyCoin_1:
	pop de
	call ClipSprite_MASK
	ret

drawMagicMushroom:
	ld a,(ix+state)
	or a \ jr nz, drawMagicMushroom_growing
	pop de
	ld hl,MagicMushroom
	call ClipSprite_MASK
	ret

drawMagicMushroom_growing:
	sra a
	sra a
	ld b,a
	ld a,8
	sub b
	pop de
	ld hl,MagicMushroom+1
	inc d
	call ClipSprite_MASK_aisheight
	ret



;----------------------------------------------------------
; playerDraw
;----------------------------------------------------------
playerDraw:
	ld hl,playerIsHit
	ld a,(hl)
	or a \ jr z,playerDraw_2
	dec (hl)
	bit 1,a \ ret nz
playerDraw_2:

	ld ix,dataObjects
	call objectGetScreenPos

	push de  ;coords -> stack

	ld hl,dataObjects
	push hl
	pop ix
	ld a,(ix+state)

	or a \ jr z,playerDead
	cp 1 \ jr z,playerSmall

playerDead:
playerBigStand:
	pop hl
	ret

;-----------------------------
; playerSmall
;-----------------------------
playerSmall:
	ld a,(dataObjects+16)
	or a \ jr nz, playerBig

	ld a,(ix+yspeed)
	bit 7,a \ jr nz,playerSmallJump

	;if (NOT playerInAir) AND playerIsBraking
	;   goto playerSmallBrake

	ld a,(playerInAir)
	cpl
	ld b,a
	ld a,(playerIsBraking)
	and b
	or a \ jr nz,playerSmallBrake

	;=== small mario, standing
	ld hl,MarioSmall_right1	;determine sprite
	ld a,(playerDirection)
	or a \ jr nz,pS_2
	ld hl,MarioSmall_left1	;do mirrored sprite
pS_2:	ld a,(playerWalkCycle)
	bit 1,a \ jr z,pS_3
	ld de,38
	add hl,de
pS_3:	pop de		;get coords
	call ClipSprite_MASK
	ret

playerSmallJump:
	;== small mario, jumping
	ld hl,MarioSmallJump_right	;determine sprite
	ld a,(playerDirection)
	or a \ jr nz,pS_4
	ld hl,MarioSmallJump_left	;do mirrored sprite
pS_4:	pop de
	call ClipSprite_MASK
	ret

playerSmallBrake:
	;== small mario, braking
	ld hl,MarioSmallBrake_right
	ld a,(playerDirection)
	or a \ jr nz,pS_5
	ld hl,MarioSmallBrake_left
ps_5:	pop de
	call ClipSprite_MASK
	ret

;-----------------------------
; playerBig
;-----------------------------
playerBig:
	ld a,(playerIsDucking)
	or a \ jp nz, playerDucks

	ld a,(ix+yspeed)
	bit 7,a \ jr nz,playerBigJump

	ld a,(playerInAir)
	cpl
	ld b,a
	ld a,(playerIsBraking)
	and b
	or a \ jr nz,playerBigBrake

	;=== big mario, standing
	ld hl,MarioBig_right1
	ld a,(playerDirection)
	or a \ jr nz,pB_2
	ld hl,MarioBig_left1
pB_2:
	ld de,0
	ld a,(playerWalkCycle)
	and %00000110
	rrca
	or a \ jr z,pB_3
	cp 2 \ jr z,pB_6
	ld de,62
	jr pB_3
pB_6:	ld de,124
pB_3:
	add hl,de
	pop de		;get coords
	call ClipSprite_MASK

	call playerBigTestsprite
	ret

playerBigJump:
	;== small mario, jumping
	ld hl,MarioBigJump_right	;determine sprite
	ld a,(playerDirection)
	or a \ jr nz,pB_4
	ld hl,MarioBigJump_left	;do mirrored sprite
pB_4:	pop de
	call ClipSprite_MASK

	call playerBigTestsprite
	ret

playerBigBrake:
	;== big mario, braking
	ld hl,MarioBigBrake_right
	ld a,(playerDirection)
	or a \ jr nz,pB_5
	ld hl,MarioBigBrake_left
pB_5:	pop de
	call ClipSprite_MASK

	call playerBigTestsprite
	ret

playerBigTestsprite:
	ret
	ld ix,dataObjects+16
	call objectGetScreenPos
	ld hl,TestSprite
	call ClipSprite_MASK
	ret

playerDucks:
	;== big mario, ducking
	ld hl,MarioDuck_right
	ld a,(playerDirection)
	or a \ jr nz,pB_7
	ld hl,MarioDuck_left
pB_7:	pop de
	call ClipSprite_MASK

	call playerBigTestsprite
	ret



