;==========================================================
; prepareSprites
;==========================================================
; mirrors sprites
prepareSprites:
	ld hl,MarioSmall_right1 ;source
	call mirrorSprite
	ld hl,MarioSmall_right2
	call mirrorSprite
	ld hl,MarioSmallJump_right
	call mirrorSprite
	ld hl,MarioSmallBrake_right
	call mirrorSprite

	ld hl,MarioBig_right1
	call mirrorSprite
	ld hl,MarioBig_right2
	call mirrorSprite
	ld hl,MarioBig_right3
	call mirrorSprite
	ld hl,MarioBigJump_right
	call mirrorSprite
	ld hl,MarioBigBrake_right
	call mirrorSprite

	ld hl,MarioDuck_right
	call mirrorSprite

	ld hl,Goomba_1
	call mirrorSprite

	ld hl,KoopaTroopa_right1
	call mirrorSprite
	ld hl,KoopaTroopa_right2
	call mirrorSprite
	
	ret

; mirrorSprites (hl=source data)
; target comes 'after' source
mirrorSprite:
	ld a,(hl)
	add a,a
	ld b,a
	inc a
	ld d,0
	ld e,a
	inc hl
	push hl
	add hl,de
	pop de

ms_lp:	ld a,(de)
	ld c,0
	rlca
	rr c
	rlca
	rr c
	rlca
	rr c
	rlca
	rr c
	rlca
	rr c
	rlca
	rr c
	rlca
	rr c
	rlca
	rr c
	ld a,c
	ld (hl),a
	inc hl
	inc de
	djnz ms_lp
	ret


;==========================================================
; powerblock_check
;==========================================================
; INPUT: DE coordinates
; OUTPUT: action performed if powerblock
;         a=255 if no powerblock
powerblock_check:
	push hl
	push de
	ld hl,powerblockmem
	ld a,(hl)
	or a \ ret z
	ld b,a
	inc hl
pbc_lp: ld a,(hl)
	inc hl
	cp d \ jr nz, pbc_nx
	ld a,(hl)
	cp e \ jr z, powerblock_hit
pbc_nx: inc hl
	inc hl
	djnz pbc_lp
	pop de
	pop hl
	ld a,255
	ret

;powerblock is SMASHED!!!
;==========================================================
; powerblock_hit
;==========================================================
; INPUT: (HL+1) = powerblock nr
;        DE = coordinates on map
powerblock_hit:
	pop de
	inc hl
	ld a,(hl)       ;a  = powerblock nr
	pop hl		;hl = map loc
	cp 101 \ jr z, powerblock_magicmushroom
	ret

bumptile:
	call objectFindFreeSpot
	ld (hl), JUMPY_TILE
	inc hl             	;x0
	inc hl \ ld (hl),d      ;x1
	inc hl \ dec (hl)       ;y0
	inc hl \ ld (hl),e      ;y1
	inc hl                  ;origx
	inc hl                  ;origy
	inc hl		        ;xspeed
	inc hl			;yspeed
	inc hl \ ld (hl),3      ;state (frames left)
	inc hl
	ret

;------- magic mushroom
powerblock_magicmushroom:
	ld (hl),50
	call bumptile
	  ld (hl),0
	call objectFindFreeSpot
	  ld (hl), MAGIC_MUSHROOM
	  ld b,0
	  ld c,31
	  dec e	;Y-1
	  call setVillain
	  ld (hl),0
	ret

	.db "boomboom"

;==========================================================
; powerblock_insert
;==========================================================
; INPUT: A value
;        DE coordinates
; inserts slot in powerblock table
powerblock_insert:
	push af
	ld hl,powerblockmem
	ld a,(hl)
	ld c,a
	add a,c
	add a,c
	inc (hl)
	inc hl
	ld b,0
	ld c,a
	add hl,bc
	ld (hl),d
	inc hl
	ld (hl),e
	inc hl
	pop af
	ld (hl),a
	ret

	.db "herthert"

;==========================================================
;  loosebrickDraw
;==========================================================
loosebrickDraw:
	ld ix,loosebrickmem-3
	call losebrickDraw_dotwo
	ld ix,loosebrickmem+9
	call losebrickDraw_dotwo
	ret

losebrickDraw_dotwo:
	ld a,(ix+3)
	or a \ ret z

	call addgravity
	ld l,(ix+6)
	ld h,(ix+7)
	ld d,0
	or a
	jp m, loosebrickDraw_1
	ld e,a
	add hl,de
	jr loosebrickDraw_2
loosebrickDraw_1:
	cpl
	inc a
	ld e,a
	or a
	sbc hl,de
loosebrickDraw_2:
	ld (ix+6),l
	ld (ix+7),h
	ld b,h
	ld a,(screenHeight)
	cp b \ jp z,killloosebrick
	ld (ix+12),l
	ld (ix+13),h


	ld de,48
	ld l,(ix+4)
	ld h,(ix+5)
	or a
	sbc hl,de
	ld (ix+4),l
	ld (ix+5),h
	ld l,(ix+10)
	ld h,(ix+11)
	add hl,de
	ld (ix+10),l
	ld (ix+11),h

	inc ix
	inc ix
	inc ix
	push ix
	call objectGetScreenPos
	ld hl,JumpyBrick
	call ClipSprite_MASK
	pop hl
	ld de,6
	add hl,de
	push hl
	pop ix
	call objectGetScreenPos
	ld hl,JumpyBrick
	call ClipSprite_MASK

	ret

killloosebrick:
	ld (ix+3),0
	ld (ix+4),0
	ld (ix+6),0
	ret

	.db "hayehaye"

;==========================================================
;  setVillain
;==========================================================
; DE = X,Y
; B=xspeed
; C=state
setVillain:
	inc hl		      	;x0
	inc hl \ ld (hl),d      ;x1
	inc hl \ ld (hl),255    ;y0
	inc hl \ ld (hl),e      ;y1
	inc hl \ ld (hl),d      ;origx
	inc hl \ ld (hl),e      ;origy
	inc hl \ ld (hl),b      ;xspeed
	inc hl                  ;yspeed
	inc hl \ ld (hl),c      ;state
	inc hl
	ret


;==========================================================
; wait
;==========================================================
; BC = time
wait:	push bc
	ld b,255
wait2:	push ix
	pop ix
	push ix
	pop ix
	djnz wait2
	pop bc
	djnz wait

;==========================================================
; gsGetK
;==========================================================
gsGetK:
	push bc
	push de
	push hl
gs_GetK2:
	ld b,7
gs_GetK_loop:

	ld a,7
	sub b
	ld hl,gs_keygroups
	ld d,0 \ ld e,a
	add hl,de
	ld a,(hl)
	ld c,a

	ld a,0ffh           ;resets the keypad.
	out (1),a           ;'Required syntax' :)
	ld a,c
	out (1),a
	in a,(1)            ;'Required Syntax'

	ld de,0
	cp 254 \ jr z,gs_GetK_254
	cp 253 \ jr z,gs_GetK_253
	cp 251 \ jr z,gs_GetK_251
	cp 247 \ jr z,gs_GetK_247
	cp 239 \ jr z,gs_GetK_239
	cp 223 \ jr z,gs_GetK_223
	cp 191 \ jr z,gs_GetK_191
	cp 127 \ jr z,gs_GetK_127

gs_GetK_loopend:

	djnz gs_GetK_loop

	xor a
	ld (gs_keymem),a
	jr gs_GetK_end

gs_GetK_127:
	inc e
gs_GetK_191:
	inc e
gs_GetK_223:
	inc e
gs_GetK_239:
	inc e
gs_GetK_247:
	inc e
gs_GetK_251:
	inc e
gs_GetK_253:
	inc e
gs_GetK_254:

	push de
	ld a,7
	sub b
	add a,a \ add a,a \ add a,a
	ld d,0 \ ld e,a
	ld hl,gs_keygroup1
	add hl,de
	pop de
	add hl,de
	ld a,(hl)

	ld d,a
	ld a,(gs_keymem)
	cp d \ jr z,gs_getK_end
	ld a,d
	ld (gs_keymem),a

gs_GetK_end:

	pop hl
	pop de
	pop bc
	ret

gs_keygroups:
	.db $fe, $fd, $fb, $f7, $ef, $df, $bf
gs_keygroup1:
	.db 01h, 02h, 03h, 04h, 00h, 00h, 00h, 00h
gs_keygroup2:
	.db 09h, 0Ah, 0Bh, 0Ch, 0Dh, 00h, 0Fh, 00h
gs_keygroup3:
	.db 8Ch, 12h, 13h, 14h, 15h, 16h, 17h, 00h
gs_keygroup4:
	.db 8Dh, 1Ah, 1Bh, 1Ch, 1Dh, 1Eh, 1Fh, 20h
gs_keygroup5:
	.db 21h, 22h, 23h, 24h, 25h, 26h, 27h, 28h
gs_keygroup6:
	.db 00h, 2Ah, 2Bh, 2Ch, 2Dh, 2Eh, 2Fh, 30h
gs_keygroup7:
	.db 31h, 32h, 33h, 34h, 35h, 36h, 37h, 38h 
gs_keymem:
	.db 0









